@@ -2,214 +2,9 @@ package HOI::Comprehensions;
 
 require Exporter;
 our @ISA = qw( Exporter );
-our @EXPORT_OK = qw( comp get_member get_list is_over );
-our $VERSION = '0.034';
+our @EXPORT_OK = qw( comp );
+our $VERSION = '0.04';
 
-#use Alias qw(attr);
-
-# The inline code below comes from Alias (https://metacpan.org/pod/Alias) with a patch from issue #64987 (https://rt.cpan.org/Public/Bug/Display.html?id=64987)
-
-use Inline C => <<'END_C';
-
-#ifndef PERL_VERSION
-#include "patchlevel.h"
-#define PERL_REVISION         5
-#define PERL_VERSION          PATCHLEVEL
-#define PERL_SUBVERSION       SUBVERSION
-#endif
-
-#ifndef GvCV_set
-#define GvCV_set(gv,cv) GvCV((gv)) = (cv)
-#endif
-
-#if PERL_REVISION == 5 && (PERL_VERSION < 4 || (PERL_VERSION == 4 && PERL_SUBVERSION <= 75 ))
-#define PL_stack_sp	stack_sp
-#endif
-
-void process_flag(char *varname, SV **svp, char **strp, STRLEN *lenp)
-{
-    GV *vargv = gv_fetchpv(varname, FALSE, SVt_PV);
-    SV *sv = Nullsv;
-    char *str = Nullch;
-    STRLEN len = 0;
-
-    if (vargv && (sv = GvSV(vargv))) {
-	if (SvROK(sv)) {
-	    if (SvTYPE(SvRV(sv)) != SVt_PVCV)
-		croak("$%s not a subroutine reference", varname);
-	}
-	else if (SvOK(sv))
-	    str = SvPV(sv, len);
-    }
-    *svp = sv;
-    *strp = str;
-    *lenp = len;
-}
-
-void attr(SV *hashref)
-{
-    dSP;
-    HV *hv;
-    int in_destroy = 0;
-    int deref_call;
-    
-    if (SvREFCNT(hashref) == 0)
-    in_destroy = 1;
-    
-    ++SvREFCNT(hashref);	/* in case LEAVE wants to clobber us */
-
-    if (SvROK(hashref) &&
-    (hv = (HV *)SvRV(hashref)) && (SvTYPE(hv) == SVt_PVHV))
-    {
-    SV *val, *tmpsv;
-    char *key;
-    I32 klen;
-    SV *keypfx, *attrpfx, *deref;
-    char *keypfx_c, *attrpfx_c, *deref_c;
-    STRLEN keypfx_l, attrpfx_l, deref_l;
-
-    //process_flag("Alias::KeyFilter", &keypfx, &keypfx_c, &keypfx_l);
-    (keypfx = NULL), (keypfx_c = NULL), (keypfx_l = 0);
-    process_flag("HOI::AttrPrefix", &attrpfx, &attrpfx_c, &attrpfx_l);
-    //process_flag("Alias::Deref", &deref, &deref_c, &deref_l);
-    (deref = NULL), (deref_c = NULL),(deref_l = 0);
-    deref_call = (deref && !deref_c);
-    
-    LEAVE;                      /* operate at a higher level */
-    
-    (void)hv_iterinit(hv);
-    while ((val = hv_iternextsv(hv, &key, &klen))) {
-        GV *gv;
-        CV *cv;
-        int stype = SvTYPE(val);
-        int deref_this = 1;
-        int deref_objects = 0;
-
-        /* check the key for validity by either looking at
-         * its prefix, or by calling &$Alias::KeyFilter */
-        if (keypfx) {
-            if (keypfx_c) {
-                if (keypfx_l && klen > keypfx_l
-                && strncmp(key, keypfx_c, keypfx_l))
-                continue;
-            } else {
-                //dSP;
-                SV *ret = Nullsv;
-                I32 i;
-                
-                ENTER; SAVETMPS; PUSHMARK(sp);
-                XPUSHs(sv_2mortal(newSVpv(key,klen)));
-                PUTBACK;
-                if (perl_call_sv(keypfx, G_SCALAR))
-                ret = *PL_stack_sp--;
-                SPAGAIN;
-                i = SvTRUE(ret);
-                FREETMPS; LEAVE;
-                if (!i)
-                continue;
-            }
-        }
-
-        if (SvROK(val) && deref) {
-            if (deref_c) {
-                if (deref_l && !(deref_l == 1 && *deref_c == '0'))
-                deref_objects = 1;
-            }
-            else {
-                //dSP;
-                SV *ret = Nullsv;
-                
-                ENTER; SAVETMPS; PUSHMARK(sp);
-                XPUSHs(sv_2mortal(newSVpv(key,klen)));
-                XPUSHs(sv_2mortal(newSVsv(val)));
-                PUTBACK;
-                if (perl_call_sv(deref, G_SCALAR))
-                ret = *PL_stack_sp--;
-                SPAGAIN;
-                deref_this = SvTRUE(ret);
-                FREETMPS; LEAVE;
-            }
-        }
-        
-        /* attributes may need to be prefixed/renamed */
-        if (attrpfx) {
-            STRLEN len;
-            if (attrpfx_c) {
-                if (attrpfx_l) {
-                    SV *keysv = sv_2mortal(newSVpv(attrpfx_c, attrpfx_l));
-                    sv_catpvn(keysv, key, klen);
-                    key = SvPV(keysv, len);
-                    klen = len;
-                }
-            }
-            else {
-                //dSP;
-                SV *ret = Nullsv;
-                
-                ENTER; PUSHMARK(sp);
-                XPUSHs(sv_2mortal(newSVpv(key,klen)));
-                PUTBACK;
-                if (perl_call_sv(attrpfx, G_SCALAR))
-                ret = *PL_stack_sp--;
-                SPAGAIN; LEAVE;
-                key = SvPV(ret, len);
-                klen = len;
-            }
-        }
-
-        if (SvROK(val) && (tmpsv = SvRV(val))) {
-        if (deref_call) {
-            if (!deref_this)
-            goto no_deref;
-        }
-        else if (!deref_objects && SvOBJECT(tmpsv))
-            goto no_deref;
-
-        stype = SvTYPE(tmpsv);
-        if (stype == SVt_PVGV)
-            val = tmpsv;
-
-        }
-        else if (stype != SVt_PVGV) {
-        no_deref:
-        val = sv_2mortal(newRV(val));
-        }
-        
-        /* add symbol, forgoing "used once" warnings */
-        gv = gv_fetchpv(key, GV_ADDMULTI, SVt_PVGV);
-        
-        switch (stype) {
-        case SVt_PVAV:
-        save_ary(gv);
-        break;
-        case SVt_PVHV:
-        save_hash(gv);
-        break;
-        case SVt_PVGV:
-        save_gp(gv,TRUE);   /* hide previous entry in symtab */
-        break;
-        case SVt_PVCV:
-            cv = GvCV(gv);
-        SAVESPTR(cv);
-        GvCV_set(gv,Null(CV*));
-        break;
-        default:
-        save_scalar(gv);
-        break;
-        }
-        sv_setsv((SV*)gv, val); /* alias the SV */
-    }
-    ENTER;			    /* in lieu of the LEAVE far beyond */
-    }
-    if (in_destroy)
-    --SvREFCNT(hashref);	    /* avoid calling DESTROY forever */
-    else
-    SvREFCNT_dec(hashref);
-    
-    XPUSHs(hashref);                /* simply return what we got */
-}
-
-END_C
 
 sub comp {
     my $computation = shift;
@@ -316,21 +111,32 @@ sub step_next_lazy {
     my ($done, $arguments) = eval $self->{geneitr};
     $self->{all_done} = $done;
     my ($package_name) = $self->{caller};
-    local $HOI::AttrPrefix = $package_name.'::';
-    attr $arguments;
-    %switches = (
+    local $AttrPrefix = $package_name.'::';
+    my $evalstr = '';
+    for my $key (keys %$arguments) {
+        $evalstr .= 'local $'."$AttrPrefix"."$key".' = $arguments->{'."$key".'}; ';
+    }
+    my %switches = (
         full => sub { 
             my $guards_ok = 1; 
-            ($_->($arguments) or $guards_ok = 0) for (@{$self->{guards}}); 
-            push @{$self->{list}}, $self->{computation}->($arguments) if ($guards_ok); 
+            eval '{'.$evalstr.'($_->($arguments) or $guards_ok = 0) for (@{$self->{guards}}); '.'push @{$self->{list}}, $self->{computation}->($arguments) if ($guards_ok); '.'}';
             $guards_ok 
         },
-        #cdr => sub { $self->{computation}->($arguments) },
     );
     my $guard = $switches{$flag}->();
     ($self->{list}, $done, $guard)
 }
 
+sub next { 
+    my ($l_, $done, $guard);
+    for my $cnt (0..$_[1]) {
+        do {
+            ($l_, $done, $guard) = $_[0]->step_next_lazy('full'); 
+        } until ($done or $guard);
+    }
+    ($l_->[$#$l_], $done, $guard)
+}
+
 sub next { 
     my ($l_, $done, $guard);
     #print "cnt to $_[1]\n";
@@ -362,9 +168,9 @@ HOI::Comprehensions - Higher-Order Imperative "Re"features in Perl: List Compreh
 
 =head1 SYNOPSIS
 
-  use HOI::Comprehensions qw( comp );
+  use HOI::Comprehensions;
 
-  my $list = comp( sub { $x + $y + $z }, x => [ 1, 2, 3 ], y => [ 4, 5, 6 ], z => sub { ( 1, 1 ) } )->( sub { $x > 1 } );
+  my $list = HOI::Comprehensions::comp( sub { $x + $y + $z + $w }, x => [ 1, 2, 3 ], y => [ 4, 5, 6 ], w => HOI::Comprehensions::comp( sub { $u }, u => [ 1, 2, 3 ] )->(), z => sub { (2, 1) } )->( sub { $x > 1 } );
 
   my ($elt, $done);
   do {
@@ -380,8 +186,8 @@ generators of an infinite list. It works as if evaluating multi-level loops lazi
 
 Currently, the generators are handled in sequence of the argument list offered by the user.
 As a result of such implementation, a list
-    { (x, y) | x is natural number, y belongs to { 0, 1 }, x is odd }
-may be evaluated incorrectly. To avoid such situation, make sure finite generators 
+    { (x, y) | x belongs to { 0, 1 }, y belongs to natural number set }
+may be trapped in the inner infinite loop. To avoid such situation, make sure finite generators 
 are subsequent to all the infinite ones.
 
 =head1 FUNCTIONS
@@ -389,8 +195,13 @@ are subsequent to all the infinite ones.
 =head2 comp($@)->(@)
 
 For creating a list comprehension object. The formula for computing the elements of
-the list is given as a subroutine, following by the generators, in form of name => arrayref
-or name => subroutine. Comp returns a function which takes all guards in form of subroutines.
+the list is given as a subroutine, following by the generators, in form of name => arrayref, 
+name => subroutine or name => comprehension. Comp returns a function which takes all guards 
+in form of subroutines. Guard parameters can be left empty if there is no guard.
+
+The variable names for naming the ganerators could be used directly in the computation sub and
+all guard subs without strict vars enabled. They have local scope as if they were declared with
+keyword 'local' in Perl.
 
 A hashref which holds generator variables as its keys and value of those variables as
 its values is passed to the formula subroutine. However, it is recommended to use such
@@ -400,6 +211,8 @@ Generators can be arrayrefs, subroutines or list comprehensions. A subroutine ge
 return a pair ( elt, done ), where elt is the next element and done is a flag telling whether
 the iteration is over.
 
+The subroutine comp is in EXPORT_OK, but it is not exported by default
+
 =head1 METHODS
 
 =head2 get_list
